<?php
	require_once IWP_PLUGIN_PATH . 'includes/iwpPluginOptions.php';
	require_once IWP_PLUGIN_PATH . 'admin/includes/iwpCustomEvents.php';
	require_once IWP_PLUGIN_PATH . 'admin/includes/iwpAdminUtils.php';

	class iwpWhatsAppChatController {

		/**
		 * Constructor
		 */
		public function __construct() {
			wp_enqueue_style('indigitall-whatsAppChat-styles', IWP_ADMIN_URL . 'views/whatsAppChat/css/iwp-whatsAppChat-styles.css');
			wp_enqueue_script('indigitall-whatsAppChat-scripts', IWP_ADMIN_URL . 'views/whatsAppChat/js/iwp-whatsAppChat-scripts.js');
			wp_enqueue_script('indigitall-whatsAppChat-step1-scripts', IWP_ADMIN_URL . 'views/whatsAppChat/js/iwp-whatsAppChat-step1-scripts.js');
			wp_enqueue_script('indigitall-whatsAppChat-step2-scripts', IWP_ADMIN_URL . 'views/whatsAppChat/js/iwp-whatsAppChat-step2-scripts.js');
			wp_enqueue_script('indigitall-whatsAppChat-step3-scripts', IWP_ADMIN_URL . 'views/whatsAppChat/js/iwp-whatsAppChat-step3-scripts.js');
		}

		public function renderHtml() {
			$status = ((get_option(iwpPluginOptions::WH_STATUS, '0') === '1') ? 'enabled' : 'disabled');

			$whStep1View = $this->loadStep1View();
			$whStep2View = $this->loadStep2View();
			$whStep3View = $this->loadStep3View();

			ob_start();
			include_once IWP_ADMIN_PATH . 'views/whatsAppChat/iwpWhatsAppChatView.php';
			return ob_get_clean();
		}

		/***** FUNCIONES SECUNDARIAS *****/

		private function loadStep1View() {
			$phoneCountryPrefix = get_option(iwpPluginOptions::WH_PHONE_COUNTRY, false);
			$countriesPrefixOptions = iwpAdminUtils::loadCountriesPrefixOptions($phoneCountryPrefix);
			$phone = substr(get_option(iwpPluginOptions::WH_PHONE, false), strlen($phoneCountryPrefix));

			$paramsStep1 = array(
				'countriesPrefixOptions' => $countriesPrefixOptions,
				'phone'                  => $phone,
				'welcomeMessage'         => self::encodeDecodeText(get_option(iwpPluginOptions::WH_CHAT_WELCOME_MESSAGE, false)),
			);
			return iwpAdminUtils::loadViewToVar(IWP_ADMIN_PATH . 'views/whatsAppChat/partials/iwpWhatsAppChatStep1.php', $paramsStep1);
		}

		private function loadStep2View() {
			$iconSleep = get_option(iwpPluginOptions::WH_ICON_SLEEP, false);
			$iconSleep = $iconSleep ? (int)$iconSleep : 5;
			$paramsStep2 = array(
				'iconPosition'      => get_option(iwpPluginOptions::WH_ICON_POSITION, false),
				'iconColor'         => get_option(iwpPluginOptions::WH_ICON_COLOR, false),
				'iconImageId'       => get_option(iwpPluginOptions::WH_ICON_IMAGE_ID, false),
				'iconImage'         => get_option(iwpPluginOptions::WH_ICON_IMAGE_NAME, false),
				'iconTransparent'   => get_option(iwpPluginOptions::WH_ICON_TRANSPARENT_COLOR, '0'),
				'iconBalloon'       => get_option(iwpPluginOptions::WH_ICON_SPEECH_BALLOON, false),
				'iconBalloonText'   => self::encodeDecodeText(get_option(iwpPluginOptions::WH_ICON_SPEECH_BALLOON_TEXT, '')),
				'iconSleep'         => $iconSleep,
			);
			return iwpAdminUtils::loadViewToVar(IWP_ADMIN_PATH . 'views/whatsAppChat/partials/iwpWhatsAppChatStep2.php', $paramsStep2);
		}

		private function loadStep3View() {
			$chatSleep = get_option(iwpPluginOptions::WH_CHAT_SLEEP, false);
			$chatSleep = $chatSleep ? (int)$chatSleep : 20;
			$paramsStep3 = array(
				'chatHeaderValue'     => self::encodeDecodeText(get_option(iwpPluginOptions::WH_CHAT_HEADER, '')),
				'chatBodyValue'       => self::encodeDecodeText(get_option(iwpPluginOptions::WH_CHAT_BODY, '')),
				'themeColor'          => get_option(iwpPluginOptions::WH_CHAT_COLOR, false),
				'chatButtonTextValue' => self::encodeDecodeText(get_option(iwpPluginOptions::WH_CHAT_BUTTON_TEXT, '')),
				'buttonIcon'          => get_option(iwpPluginOptions::WH_CHAT_BUTTON_IMAGE_ID, false),
				'chatSleep'           => $chatSleep,
			);
			return iwpAdminUtils::loadViewToVar(IWP_ADMIN_PATH . 'views/whatsAppChat/partials/iwpWhatsAppChatStep3.php', $paramsStep3);
		}

		/***** FUNCIONES AJAX *****/

		public static function saveWhatsAppChatAjax() {
			$defaultColor = '#00bb2d'; // Definimos un color predeterminado por si acaso
			$status = (get_option(iwpPluginOptions::WH_STATUS, '0') === '1');

			// Default texts
			$welcomeMessageDefault  = __('Hello, I have a question', 'iwp-text-domain');
			$iconBalloonTextDefault = __('Hi', 'iwp-text-domain');
			$headerDefault          = __('iurny.com', 'iwp-text-domain');
			$welcomeDefault         = __("Welcome to iurny's chat", 'iwp-text-domain');
			$buttonDefault          = __('Open chat', 'iwp-text-domain');

			$phone               = iwpAdminUtils::getPOSTParam('phone', false);
			$prefix              = iwpAdminUtils::getPOSTParam('countriesPrefixOptions', false);
			$welcomeMessage      = trim(iwpAdminUtils::getPOSTParam('welcomeMessage', $welcomeMessageDefault));
			$iconPosition        = trim(iwpAdminUtils::getPOSTParam('iconPosition', 'r'));
			$iconColor           = trim(iwpAdminUtils::getPOSTParam('iconColor', $defaultColor));
			$iconImageId         = (int)iwpAdminUtils::getPOSTParam('iconImageId', false);
			$iconImage           = iwpAdminUtils::getPOSTParam('iconImage', 'whatsApp');
			$iconTransparent     = filter_var(iwpAdminUtils::getPOSTParam('iconTransparent', false), FILTER_VALIDATE_BOOLEAN);
			$iconBalloon         = trim(iwpAdminUtils::getPOSTParam('iconBalloon', false));
			$iconBalloonText     = trim(iwpAdminUtils::getPOSTParam('iconBalloonText', $iconBalloonTextDefault));
			$iconSleep           = (int)iwpAdminUtils::getPOSTParam('iconSleep', 5);
			$chatHeaderValue     = trim(iwpAdminUtils::getPOSTParam('chatHeaderValue', $headerDefault));
			$chatBodyValue       = trim(iwpAdminUtils::getPOSTParam('chatBodyValue', $welcomeDefault));
			$themeColor          = trim(iwpAdminUtils::getPOSTParam('themeColor', $defaultColor));
			$chatButtonTextValue = trim(iwpAdminUtils::getPOSTParam('chatButtonTextValue', $buttonDefault));
			$buttonIcon          = iwpAdminUtils::getPOSTParam('buttonIcon', 'send');
			$chatSleep           = (int)trim(iwpAdminUtils::getPOSTParam('chatSleep', 20));

			if ($status && (empty($prefix) || !preg_match('/^\d{7,}$/', $phone))) {
				return json_encode(array(
					'status' => 0,
					'message' => __("The telephone number or the international prefix are not correct", 'iwp-text-domain')
				));
			}

			update_option(iwpPluginOptions::WH_PHONE,                    ($prefix.$phone));
			update_option(iwpPluginOptions::WH_PHONE_COUNTRY,            $prefix);
			update_option(iwpPluginOptions::WH_CHAT_WELCOME_MESSAGE,     self::encodeDecodeText($welcomeMessage));
			update_option(iwpPluginOptions::WH_ICON_POSITION,            $iconPosition);
			update_option(iwpPluginOptions::WH_ICON_COLOR,               $iconColor);
			update_option(iwpPluginOptions::WH_ICON_IMAGE_ID,            $iconImageId);
			update_option(iwpPluginOptions::WH_ICON_IMAGE_NAME,          $iconImage);
			update_option(iwpPluginOptions::WH_ICON_TRANSPARENT_COLOR,   $iconTransparent);
			update_option(iwpPluginOptions::WH_ICON_SPEECH_BALLOON,      $iconBalloon);
			update_option(iwpPluginOptions::WH_ICON_SPEECH_BALLOON_TEXT, self::encodeDecodeText($iconBalloonText));
			update_option(iwpPluginOptions::WH_ICON_SLEEP,               $iconSleep);
			update_option(iwpPluginOptions::WH_CHAT_HEADER,              self::encodeDecodeText($chatHeaderValue));
			update_option(iwpPluginOptions::WH_CHAT_BODY,                self::encodeDecodeText($chatBodyValue));
			update_option(iwpPluginOptions::WH_CHAT_COLOR,               $themeColor);
			update_option(iwpPluginOptions::WH_CHAT_BUTTON_TEXT,         self::encodeDecodeText($chatButtonTextValue));
			update_option(iwpPluginOptions::WH_CHAT_BUTTON_IMAGE_ID,     $buttonIcon);
			update_option(iwpPluginOptions::WH_CHAT_SLEEP,               $chatSleep);

			iwpCustomEvents::sendCustomEvent(iwpCustomEvents::MACRO_WP_ACTUALIZAR);

			return json_encode(array(
				'status' => 1,
				'message' => __("WhatsApp Chat settings have been saved successfully", 'iwp-text-domain')
			));
		}

		private static function encodeDecodeText($text = '') {
			return self::decodeText(self::encodeText(stripslashes($text)));
		}

		private static function encodeText($text = '') {
			return htmlspecialchars($text, ENT_QUOTES, 'UTF-8');
		}

		private static function decodeText($text = '') {
			return htmlspecialchars_decode($text, ENT_QUOTES);
		}
	}